from random import shuffle
import logging

from pyschieber.deck import Deck
from pyschieber.rules.stich_rules import stich_rules, card_allowed
from pyschieber.rules.count_rules import count_stich, counting_factor
from pyschieber.stich import PlayedCard

logger = logging.getLogger(__name__)


class Game:
    def __init__(self, teams=None, point_limit=2500):
        self.teams = teams
        self.point_limit = point_limit
        self.players = teams[0].players + teams[1].players
        self.trumpf = None
        self.deck = Deck()
        self.stiche = []

    def play(self, start_player_index=0):
        shuffle(self.deck.cards)
        self.deal_cards()
        self.trumpf = self.players[start_player_index].choose_trumpf()
        logger.info('Chosen Trumpf: {0} \n'.format(self.trumpf))
        for i in range(9):
            stich = self.play_stich(start_player_index)
            self.count_points(stich, last=(i == 8))
            logger.info('\nStich: {0} \n'.format(stich.player))
            logger.info('{}{}\n'.format('-' * 180, self.trumpf))
            start_player_index = self.players.index(stich.player)
            self.stiche.append(stich)
            if self.teams[0].won(self.point_limit) or self.teams[1].won(self.point_limit):
                return True
        return False

    def deal_cards(self):
        for i, card in enumerate(self.deck.cards):
            self.players[i % 4].set_card(card=card)

    def play_stich(self, start_player_index):
        first_card = self.play_card(first_card=None, player=self.players[start_player_index])
        played_cards = [PlayedCard(player=self.players[start_player_index], card=first_card)]
        for i in get_player_index(start_index=start_player_index):
            current_player = self.players[i]
            card = self.play_card(first_card=first_card, player=current_player)
            played_cards.append(PlayedCard(player=current_player, card=card))
        stich = stich_rules[self.trumpf](played_cards=played_cards)
        return stich

    def play_card(self, first_card, player):
        is_allowed_card = False
        generator = player.choose_card()
        chosen_card = next(generator)
        while not is_allowed_card:
            is_allowed_card = card_allowed(first_card=first_card, chosen_card=chosen_card, hand_cards=player.cards,
                                           trumpf=self.trumpf)
            card = generator.send(is_allowed_card)
            chosen_card = chosen_card if card is None else card
        else:
            logger.info('Table: {0}:{1}'.format(player, chosen_card))
            player.cards.remove(chosen_card)
        return chosen_card

    def count_points(self, stich, last):
        stich_player_index = self.players.index(stich.player)
        cards = [played_card.card for played_card in stich.played_cards]
        self.add_points(team_index=(stich_player_index % 2), cards=cards, last=last)

    def add_points(self, team_index, cards, last):
        self.teams[team_index].points += count_stich(cards, self.trumpf, last=last) * counting_factor[self.trumpf]


def get_player_index(start_index):
    for i in range(1, 4):
        yield (i + start_index) % 4



import random

from pyschieber.player.base_player import BasePlayer
from pyschieber.trumpf import Trumpf


class RandomPlayer(BasePlayer):
    def choose_trumpf(self):
        return random.choice(list(Trumpf))

    def choose_card(self):
        allowed = False
        while not allowed:
            card = random.choice(self.cards)
            allowed = yield card
            if allowed:
                yield None


from pyschieber.player.base_player import BasePlayer
from pyschieber.trumpf import Trumpf


class CliPlayer(BasePlayer):
    def choose_trumpf(self):
        trumpfs = list(Trumpf)
        self._print_cards()
        print('\nTrumpf:')
        for i, trumpf in enumerate(trumpfs):
            print('{0} : {1}'.format(i, trumpf))
        while True:
            try:
                trumpf_index = int(
                    input("Please chose the trumpf by the number from 0 to {0}: \n".format(len(trumpfs) - 1)))
                if trumpf_index in range(0, len(trumpfs)):
                    return trumpfs[trumpf_index]
                else:
                    print("Sorry, no valid trumpf number.\n")
                    continue
            except ValueError:
                print("Sorry, I didn't understand that.\n")
                continue

    def choose_card(self):
        self._print_cards()
        allowed = False
        while not allowed:
            card = self._handle_input()
            allowed = yield card
            if allowed:
                yield None
            else:
                print("Card not allowed!\n")

    def _handle_input(self):
        while True:
            try:
                card_index = int(
                    input("Please chose the card by the number from 0 to {0}: \n".format(len(self.cards) - 1)))
                if card_index in range(0, len(self.cards)):
                    print()
                    return self.cards[card_index]
                else:
                    print("Sorry, no valid trumpf number\n")
                    continue
            except ValueError:
                print("Sorry, I didn't understand that.\n")
                continue

    def _print_cards(self):
        print('Hand cards: \n')
        for i, card in enumerate(self.cards):
            print('{0} : {1}'.format(i, card))
        print('')



import inspect


class BasePlayer:
    def __init__(self, name='unknown'):
        self.id = 0
        self.name = name
        self.cards = []

    def set_card(self, card):
        self.cards.append(card)

    def choose_trumpf(self):
        raise NotImplementedError(str(inspect.stack()[1][3]))

    def choose_card(self):
        raise NotImplementedError(str(inspect.stack()[1][3]))

    def __str__(self):
        return '<Player:{}>'.format(self.name)



import logging

from pyschieber.game import Game
from pyschieber.player.base_player import BasePlayer
from pyschieber.team import Team

logger = logging.getLogger(__name__)


class Tournament:
    def __init__(self, point_limit=1500):
        self.point_limit = point_limit
        self.players = []
        self.teams = []
        self.round = 1

    def check_players(self):
        player_numbers = []
        for index, player in enumerate(self.players):
            assert isinstance(player, BasePlayer)
            player_numbers.append(index)
        assert {0, 1, 2, 3} == set(player_numbers)

    def register_player(self, player):
        assert len(self.players) < 4
        self.players.append(player)

    def build_teams(self):
        self.check_players()
        team_1 = Team(players=[self.players[0], self.players[2]])
        team_2 = Team(players=[self.players[1], self.players[2]])
        self.teams += [team_1, team_2]

    def play(self):
        self.build_teams()
        logger.info('Tournament starts, the goal are {} points.'.format(self.point_limit))
        end = False
        while not end:
            game = Game(teams=self.teams, point_limit=self.point_limit)
            logger.info('-' * 200)
            logger.info('Round {} starts.'.format(self.round))
            logger.info('-' * 200)
            end = game.play()
            logger.info('Round {} is over.'.format(self.round))
            logger.info('Points: Team 1: {0} , Team 2: {1}. \n'.format(self.teams[0].points, self.teams[1].points))
            self.round += 1



from setuptools import setup, find_packages

with open('README.md') as f:
    readme = f.read()

    setup(
        name='pyschieber',
        version='0.0.3',
        description='pyschieber is a terminal application of the popular swiss card game Schieber and provides an API to the game',
        long_description=readme,
        author='Samuel Kurath',
        author_email='samuel.kurath@gmail.com',
        url='https://github.com/Murthy10/pyschieber',
        license='MIT',
        packages=find_packages(exclude=('tests', 'docs')),
        scripts=['bin/pyschieber'],
    )



class Card:
    names = {6: '6', 7: '7', 8: '8', 9: '9', 10: 'Banner', 11: 'Under', 12: 'Ober', 13: 'Koennig', 14: 'Ass'}

    def __init__(self, suit, value):
        self.suit = suit
        self.value = value

    def __eq__(self, other):
        return self.suit == other.suit and self.value == other.value

    def __hash__(self):
        return hash(str(self))

    def __str__(self):
        name = str(self.value)
        if self.value > 9:
            name = Card.names[self.value]
        return '<{0}:{1}>'.format(self.suit.name, name)
